#####Source code 1 (Revised)
#####Multiple preferred escape trajectories are explained by a geometric model incorporating preyfs turn and predator attack endpoint 
#####Yuuki Kawabata, Hideyuki Akada, Ken-ichiro Shimatani, Gregory N. Nishihara, Hibiki Kimura, Nozomi Nishiumi, Paolo Domenici
#####Revised in 2022-11-11
#### Please change encoding to "CP932" if the text shows garbled character.

#### 0. Preparation-----
### 0.1. Packages needed for analyses-----
library(ggpubr)
library(tidyverse)
library(svglite)
library(gridExtra)
library(MASS)
library(nleqslv)
library(ggplot2)
library(gamm4)
library(circular)
library(grid)
library(truncnorm)
library(fitdistrplus)
library(kuiper.2samp)
library(lme4)
library(lmerTest)
library(lmtest)
library(mclust)
library(lawstat)
library(brms)
library(showtext)
library(tidybayes)

font_files() |> 
  as_tibble() |> 
  filter(str_detect(ps_name, "Arial"))
font_add("arial", 
         regular = "arial.ttf",
         italic = "ariali.ttf",
         bold = "arialbd.ttf",
         bolditalic = "arialbi.ttf")
theme_gray(base_family = "arial") |> theme_set()
showtext_auto()

### 0.2. Letters for figure labels-----
ettext=expression(paste("ET ()"))
iotext=expression(paste("Initial orientation"," ",italic("")," ","()"))
tatext=expression(paste("Turn angle"," ",italic("")," ", "()"))
tatext2=expression(paste("Turn angle |",italic(""), "| ()"))
ttrtext=expression(paste("Time for displacement (ms)"))
vel=expression(Mean~velocity~(m~s^{-1}))

#### 1. Functions for the geometric model (Fig. 2) -----
### 1.1. Function for the modification of Domenici's model (Fig. 2B)-----

#function for calculating the angle from two vectors
angle=function(x1,x2,y1,y2){
  a1=x1*x2+y1*y2
  a2=x1*y2-x2*y1
  b=atan2(a2,a1)
  return(b)
}

#function for calculating the distance needed for the prey to exist the danger zone, 
#and the distance needed for the predator to reach the exit point. 
fdis1=function(et,Dwidth,Dattack,Rdevice){
  #et is the escape trajectory (degrees)
  #Dwidth is the half-width of the predator capture device
  #Dattack is the distance between the prey's initial position and the tip of the predator capture device at the end of the predator attack
  #Rdevice is the radius of the predator's capture device at the moment of attack, which is approximated as an arc
  
  ET=pi*et/180
  
  th2=asin(Dwidth/Rdevice)
  D2=Rdevice*(1-cos(th2))
  thco=atan2(Dwidth,(D2-Dattack))
  
  
  if(ET<=thco){ #sideway safety zone
    dis.prey=Dwidth/sin(ET)
    dis.pred=-Dwidth/tan(ET)+D2
  }else{
    
    if(ET==pi){ #special case: 180 deg. opposite to the predator (et=180)
      dis.prey=Dattack
      dis.pred=Dattack
    }else{
      if(ET==0){ #special case: directly toward the predator (et=0)
        dis.prey=+Inf
        dis.pred=-Inf
      }else{
        
        
        dis.pred=Dattack
        
        #outran safety zone toward the capture device of the predator at the end of its attack
        thmou=seq(th2,0,length.out=10000)
        xmou=Rdevice*cos(pi-thmou)+Rdevice-Dattack
        ymou=Rdevice*sin(pi-thmou)
        
        yfind=tan(ET)*xmou
        yfind2=ymou-yfind
        cross<-diff(yfind2>0)
        cp=which(abs(cross)==1)
        
        dis.prey=sqrt(xmou[cp]^2+ymou[cp]^2)
      }
      
    }
  }
  
  
  return(c(dis.prey,dis.pred))
  
}

#function for calculating the time difference of arrival at the edge of the safety zone between the prey and predator. 
predict.ET1<-function(IO,Uprey,Upred,intercept,slope1,slope2,GC,D1,Dwidth,Dattack,eamin,eamax,Rdevice){ 
  #IO is the initial orientation
  #Uprey is the prey speed
  #Upred is the predator speed
  #intercept, slope1, slope2, and GC (change point of turn angle) are from the piecewise linear regression for
  #the relationship between turn angle || and the time required for a displacement of D1
  #D1 is the displacement from the initial position of prey where it was assumed to end the fast-start phase
  #Dwidth is the half-width of the predator capture device
  #Dattack is the distance between the prey's initial position and the tip of the predator capture device at the end of the predator attack
  #eamin and eamax are the minimum and maximum values of turn angle ||
  #Rdevice is the radius of the predator's capture device at the moment of attack, which is approximated as an arc
  #IO is a vector but the other parameters are single numerical values

  #This function returns a data frame including following variables
  #"et", escape trajectory; "ea", turn angle;
  #"T1", time required for a displacement of D1 mm from the initial position of the prey;
  #"T2", time required for the prey to cover the rest of the distance to exist danger zone;
  #"Tpred", time required for the predator to reach the prey's exit point;
  #"y", Tdiff, the time difference of arrival at the edge of the safety zone between the prey and predator;
  #"io", initial orientation; "d_rank", ranking order of the Tdiff;
  #"d_dif", time difference from the optimal Tdiff;
  #"dir", directionality (con=away response, ips=toward response)

  #see Fig. 2, Table 1, and text for details of the parameters

    
    io3=numeric()
    for(i in 1:181){
      io3=c(io3,rep(i-1,541))
    }
    
    et3=rep(-180:360,181)
    
    ea3=et3-io3
    
    data3=data.frame(io3,et3,ea3)
    
    
    ETopt<-numeric() 
    max.dis<-numeric()
    
    distance <- data.frame(matrix(rep(NA, 10), nrow=1))[numeric(0), ]
    colnames(distance) <- c("et","ea", "T1","T2", "Tpred","y", "io", "d_rank","d_dif")
    
    Tpred<-function(x){
      x=abs(x)
      if(x>180){x=360-x}else{x=x}
      Tpred=fdis1(x,Dwidth,Dattack,Rdevice)[2]/Upred
      return(Tpred)
    }
    
    T1=function(xx){
      xx=abs(xx)
      if(xx<GC){
        yy=intercept+slope1*xx
      }else{
        yy=slope2*(xx-GC)+intercept+slope1*GC
      }
      return(yy)
    }
    
    T2<-function(x){
      x=abs(x)
      if(x>180){x=360-x}else{x=x}
      T2=(fdis1(x,Dwidth,Dattack,Rdevice)[1]-D1)/Uprey
      return(T2)
    }
    
    
    etw=0:180
    Trestw=sapply(etw,T2)
    Tpredw=sapply(etw,Tpred)
    T1=sapply(data3$ea3,T1)
    
    
    data=data.frame(et=et3,T2=rep(c(rev(Trestw),Trestw[-1],rev(Trestw)[-1]),181),Tpred=rep(c(rev(Tpredw),Tpredw[-1],rev(Tpredw)[-1]),181),io=io3,ea=ea3, T1)
    data=data[abs(data$ea)>=eamin & abs(data$ea)<=eamax,]
    data$y=data$Tpred-(data$T1+data$T2)
    
    data <- data %>%
      dplyr::arrange(io, y) %>%
      dplyr::group_by(io) %>%
      dplyr::mutate(d_rank = rank(y,na.last=F,ties.method="average"), d_dif=y-max(y, na.rm=T))
    
    data$d_rank[is.na(data$y)]=1
    data$dir=as.character(data$ea)
    data[data$ea*data$io<0,]$dir="ips"
    data[data$ea*data$io>=0,]$dir="con"
    
    return(data.frame(data))
    
    
  }

#example of using this function
IO=0:20
GC=44
Upred=1300
Uprey=1022
D1=15
Dwidth=18.5
Dattack=35
slope1=-0.00050/1000
slope2=0.14723/1000
intercept=27.50617/1000
Rdevice=199
eamin=0
eamax=180

out=predict.ET1(IO=IO,Uprey=Uprey,Upred=Upred,intercept=intercept,slope1=slope1,slope2=slope2,
                  GC=GC,D1=D1,Dwidth=Dwidth,Dattack=Dattack,eamin=eamin,eamax=eamax,Rdevice=Rdevice)

View(out)


### 1.2. Function for the modification of Corcoran's model (Appendix 1-Fig.1)-----

#function for calculating the distance needed for the prey to exist the danger zone, 
#and the distance needed for the predator to reach the exit point. 
fdis2=function(et,D0,Dtr,Dwidth,Dattack,Rdevice,Dreact){
  #et is the escape trajectory (degrees)
  #D0 is the distance between the prey and the predator at the onset of the prey's escape response (Dinitial in the text)
  #Dtr is the minimum turning radius of the predator (note that Dtr = Rturn (in the text) +  Dwidth)
  #Dwidth is the half-width of the predator capture device
  #Dattack is the distance between the prey's initial position and the tip of the predator capture device at the end of the predator attack
  #Rdevice is the radius of the predator's capture device at the moment of attack, which is approximated as an arc
  #Dreact is the distance required for the predator to react the prey's escape response to initiate its turn
  
  
  
  ET=pi*et/180
  
  angle=function(x1,x2,y1,y2){
    a1=x1*x2+y1*y2
    a2=x1*y2-x2*y1
    b=atan2(a2,a1)
    return(b)
  }
  
  alp=(D0-Dreact+Dattack)/Dtr
  th2=asin(Dwidth/Rdevice)
  D2=Rdevice*(1-cos(th2))
  X0=D0-Dreact+D2
  Y0=Dwidth
  X1=D2*cos(alp)+(Dwidth-Dtr)*sin(alp)+D0-Dreact
  Y1=-D2*sin(alp)+(Dwidth-Dtr)*cos(alp)+Dtr
  X2=D0-Dreact-Dtr*sin(alp)
  Y2=Dtr-Dtr*cos(alp)
  X3=D0-Dreact+Rdevice*cos(alp)-Dtr*sin(alp)
  Y3=Dtr-Rdevice*sin(alp)-Dtr*cos(alp)
  
  #change-point angle
  Ach0=angle(1,D0-Dreact+D2,0,Dwidth)
  
  if(ET<=Ach0){ #1st part: sideway safety zone before the turn
    dis.prey=Dwidth/sin(ET)
    dis.pred=-Dwidth/tan(ET)+D0+D2
  }else{
    if(ET==pi){ #special case: 180 deg. opposite to the predator (et=180)
      dis.prey=Dattack
      dis.pred=Dattack+D0
    }else{
      if(ET==0){ #special case: directly toward predator (et=0) 
        dis.prey=+Inf
        dis.pred=-Inf
      }else{
        
        #2nd part: sideway safety zone after the turn
        thside=seq(0,alp,0.00001)
        xside=D2*cos(thside)+(Dwidth-Dtr)*sin(thside)+D0-Dreact
        yside=-D2*sin(thside)+(Dwidth-Dtr)*cos(thside)+Dtr
        
        #3rd part: safety zone at the capture device
        thmou=seq(th2,0,-0.0001)
        xmou=Rdevice*cos(pi-alp-thmou)+X3
        ymou=Rdevice*sin(pi-alp-thmou)+Y3
        
        #4th part: outran safety zone depicted by an involute curve
        th=seq(0,alp,0.0001)
        xf=Dtr*(cos(th)+th*sin(th))
        yf=Dtr*(sin(th)-th*cos(th))
        xinv=D0-Dreact-sin(alp)*xf+cos(alp)*yf
        yinv=Dtr-cos(alp)*xf-sin(alp)*yf
        
        xall=c(xside,xmou,xinv)
        yall=c(yside,ymou,yinv)
        
        dfall1=data.frame(x=xall,y=yall)
        dfall2=data.frame(x=xall,y=-yall)
        
        yfind=tan(ET)*xall
        yfind2=yall-yfind
        cross<-diff(yfind2>0)
        cp=which(abs(cross)==1)
        
        if(any(cp<length(xside))){
          sidecp=cp[cp<length(xside)]
          usecp=sidecp[1]
          x1=D0-Dreact+D2-D0+Dreact
          x2=xall[usecp]-D0+Dreact
          y1=Dwidth-Dtr
          y2=yall[usecp]-Dtr
          dis.pred=Dtr*abs(angle(x1,x2,y1,y2))+Dreact
          
        }else{
          usecp=cp[length(cp)]
          dis.pred=D0+Dattack
        }
        dis.prey=sqrt(xall[usecp]^2+yall[usecp]^2)
        
        
      }
      
    } 
    
  }  
  
  return(c(dis.prey,dis.pred))
  
}

predict.ET2<-function(IO,Uprey,Upred,intercept,slope1,slope2,GC,D1,Dwidth,Dattack,eamin,eamax,D0,Dtr,Rdevice,Dreact){ 
  #IO is the initial orientation
  #Uprey is the prey speed
  #Upred is the predator speed
  #intercept, slope1, slope2, and GC (change point of turn angle) are from the piecewise linear regression for
  #the relationship between turn angle || and the time required for a displacement of D1
  #D1 is the displacement from the initial position of prey where it was assumed to end the fast-start phase
  #Dwidth is the half-width of the predator capture device
  #Dattack is the distance between the prey's initial position and the tip of the predator capture device at the end of the predator attack
  #eamin and eamax are the minimum and maximum values of turn angle ||
  #D0 is the distance between the prey and the predator at the onset of the prey's escape response (Dinitial in the text)
  #Dtr is the minimum turning radius of the predator (note that the Dtr = Rturn (in the text) +  Dwidth)
  #Rdevice is the radius of the predator's capture device at the moment of attack, which is approximated as an arc
  #Dreact is the distance required for the predator to react the prey's escape response to initiate its turn
  #IO is a vector but the other parameters are single numerical values
  
  #This function returns a data frame including following variables
  #"et", escape trajectory; "ea", turn angle;
  #"T1", time required for a displacement of D1 mm from the initial position of the prey;
  #"T2", time required for the prey to cover the rest of the distance to exist danger zone;
  #"Tpred", time required for the predator to reach the prey's exit point; 
  #"y", Tdiff, the time difference of arrival at the edge of the safety zone between the prey and predator;
  #"io", initial orientation; "d_rank", ranking order of the Tdiff; 
  #"d_dif", time difference from the optimal Tdiff; 
  #"dir", directionality (con=away response, ips=toward response)
  
  #see Appendix 1-fig.1, Table 1, and Supplementary Information Text for details of the parameters
  
  io3=numeric()
  for(i in 1:181){
    io3=c(io3,rep(i-1,541))
  }
  
  et3=rep(-180:360,181)
  
  ea3=et3-io3
  
  data3=data.frame(io3,et3,ea3)
  
  
  ETopt<-numeric() 
  max.dis<-numeric()
  
  distance <- data.frame(matrix(rep(NA, 10), nrow=1))[numeric(0), ]
  colnames(distance) <- c("et","ea", "T1","T2", "Tpred","y", "io", "d_rank","d_dif")
  
  #function for calculating the Tpred, T1, T2 in response to ET
  
  Tpred<-function(x){
    x=abs(x)
    if(x>180){x=360-x}else{x=x}
    Tpred=fdis2(x,D0,Dtr,Dwidth,Dattack,Rdevice,Dreact)[2]/Upred
    return(Tpred)
  }
  
  T1=function(xx){
    xx=abs(xx)
    if(xx<GC){
      yy=intercept+slope1*xx
    }else{
      yy=slope2*(xx-GC)+intercept+slope1*GC
    }
    return(yy)
  }
  
  T2<-function(x){
    x=abs(x)
    if(x>180){x=360-x}else{x=x}
    T2=(fdis2(x,D0,Dtr,Dwidth,Dattack,Rdevice,Dreact)[1]-D1)/Uprey
    return(T2)
  }
  
  
  etw=0:180
  Trestw=sapply(etw,T2)
  Tpredw=sapply(etw,Tpred)
  T1=sapply(data3$ea3,T1)
  
  
  data=data.frame(et=et3,T2=rep(c(rev(Trestw),Trestw[-1],rev(Trestw)[-1]),181),Tpred=rep(c(rev(Tpredw),Tpredw[-1],rev(Tpredw)[-1]),181),io=io3,ea=ea3, T1)
  data=data[abs(data$ea)>=eamin & abs(data$ea)<=eamax,]
  data$y=data$Tpred-(data$T1+data$T2)
  
  data <- data %>%
    dplyr::arrange(io, y) %>%
    dplyr::group_by(io) %>%
    dplyr::mutate(d_rank = rank(y,na.last=F,ties.method="average"), d_dif=y-max(y, na.rm=T))
  
  data$d_rank[is.na(data$y)]=1
  data$dir=as.character(data$ea)
  data[data$ea*data$io<0,]$dir="ips"
  data[data$ea*data$io>=0,]$dir="con"
  
  return(data.frame(data))
  
  
}


#example of using this function
IO=0:180
GC=44
Upred=1500
Uprey=1000
D1=15
D0=60
Dtr=1000
Dwidth=18
Dattack=35
slope1=-0.00050/1000
slope2=0.14723/1000
intercept=27.50617/1000
Rdevice=199
Dreact=40
eamin=0
eamax=180

out=predict.ET2(IO,Uprey = Uprey,Upred=Upred,slope2=slope2,slope1=slope1,
                GC=GC,intercept=intercept,D1=D1,Dwidth=Dwidth,Dattack=Dattack,
                eamin=eamin,eamax=eamax,Rdevice=Rdevice,Dreact=Dreact,D0=D0,Dtr=Dtr)

View(out)

#### 2. Experimental results (Fig. 3) -----
### 2.1. Effects of initial orientation and predator speed on ET-----
#examined in each directionality (dir; away=con, toward=ips) using the generalized additive mixed model (GAMM)

#load data
dfemp=read.csv("Dataset1.csv")
dfemp$et.plot=dfemp$et
dfemp[dfemp$et.plot<0,]$et.plot=dfemp[dfemp$et.plot<0,]$et.plot+360
dfemp = transform(dfemp, dir=factor(dir,levels = c("ips","con")))
df3=data.frame(dir=dfemp$dir,io=abs(dfemp$io),ea=dfemp$ea,et=dfemp$et,pred.sp=dfemp$pred.sp,fish=dfemp$fish)
dfcon=df3[df3$dir=="con",]
dfips=df3[df3$dir=="ips",]

#effects of io and pred.sp on et in away response
fit=gamm4(et~s(io)+s(pred.sp),random = ~ (1|fish),data=dfcon)
fit
anova(fit$gam)
anova(fit$mer)

fit=gamm4(et~s(io),random = ~ (1|fish),data=dfcon)
xx=data.frame(io=seq(0,180,by=0.01))
predcon=predict(fit$gam,xx,se.fit=TRUE, type="response")


#effects of io and pred.sp on et in toward response
fit=gamm4(et~s(io)+s(pred.sp),random = ~ (1|fish),data=dfips)
fit
anova(fit$gam)
anova(fit$mer)

fit=gamm4(et~s(io),random = ~ (1|fish),data=dfips)
xx=data.frame(io=seq(0,180,by=0.01))
predips=predict(fit$gam,xx,se.fit=TRUE, type="response")



### 2.2. Make Fig. 3-----

#xy plot for the effect of io on et (Fig. 3B)
pred2=data.frame(etcon=as.numeric(predcon$fit),etips=as.numeric(predips$fit),
                 io=as.numeric(xx$io),
                 Pu95con=as.numeric(predcon$fit + 1.96),
                 Pd95con=as.numeric(predcon$fit - 1.96),
                 Pu95ips=as.numeric(predips$fit + 1.96),
                 Pd95ips=as.numeric(predips$fit - 1.96))

fig2b=ggplot()+geom_point(data=df3, aes(x=io,y=et,fill=dir),size=1.5,shape=21)+
  ylab(ettext)+xlab(iotext)+
  geom_line(data=pred2,aes(x=io,y=etcon),size=1,linetype="solid")+
  geom_line(data=pred2,aes(x=io,y=etips),size=1,linetype="dotted")+
  theme(axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_manual(values=c("red","steelblue"))+
  scale_x_continuous(limits=c(0,180),breaks=seq(0,180,by=60))+
  scale_y_continuous(limits=c(-150,240),breaks=seq(-180,330,by=60))


#make the histogram of overall ET (Fig. 3C)
mx=0.2
mn=-0.05

H=circular(dfemp$et,type="angle",units="degrees",rotation="counter")
resx <- density(H,bw=50)
dfest=data.frame(x=seq(0,360,length=512),y=as.numeric(as.numeric(resx$y)))

fig2c=ggplot()+
  coord_polar(start=270*pi/180,direction=-1)+
  geom_histogram(data=dfemp, binwidth=15,aes(x=et.plot,y=..count../sum(..count..),
                                             fill=dir),colour="black",size=0.25,boundary=0)+
  geom_line(data=dfest,aes(x=x,y=y*0.1842865*1.5),colour="black",size=1)+
  scale_x_continuous(limits=c(0,360),breaks=seq(0,270,by=90), labels=c(0,90,180,-90))+
  scale_y_continuous(limits=c(mn,mx),breaks=seq(0,mx,by=0.1))+
  theme(axis.line=element_blank(),axis.ticks.y=element_blank(),
        axis.title.x=element_blank(),axis.text.y=element_blank(),
        axis.title.y=element_blank(),axis.text.x=element_text(size=12),
        panel.grid.major = element_line(colour="grey",),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_manual(values=c("red","steelblue"))


#make the ET histogram in each initial orientation bin (Fig 2A)
mx=0.35
mn=-0.05
ran=30
Nran=180/30

sam.bin=numeric()

for(i in 1:Nran){
  if(i*ran==180){
    df4=dfemp[dfemp$io>=(i-1)*ran & dfemp$io<=i*ran,]
  }else{
    df4=dfemp[dfemp$io>=(i-1)*ran & dfemp$io<i*ran,]
  }
  sam.bin[i]=nrow(df4)
  df4 <- transform(df4, dir=factor(dir,levels = c("con","ips")))
  H=circular(df4$et,type="angle",units="degrees",rotation="counter")
  resx <- density(H,bw=50)
  dfest=data.frame(x=seq(0,360,length=512),y=as.numeric(as.numeric(resx$y)))
  b=ggplot()+
    coord_polar(start=270*pi/180,direction=-1)+
    geom_histogram(data=df4, binwidth=15,aes(x=et.plot,y=..count../sum(..count..),
                                             fill=dir),colour="black",size=0.25,boundary=0)+
    geom_line(data=dfest,aes(x=x,y=y*0.1842865*1.5),colour="black",size=1,alpha=0.7)+
    scale_x_continuous(limits=c(0,360),breaks=seq(0,270,by=90),labels=c(0,90,180,-90))+
    scale_y_continuous(limits=c(mn,mx),breaks=seq(0,mx,by=0.1))+
    theme(plot.title=element_text(size=14,hjust=0.5),
          axis.line=element_blank(),axis.ticks.y=element_blank(),
          axis.title.x=element_blank(),axis.text.y=element_blank(),
          axis.title.y=element_blank(),axis.text.x=element_text(size=12),
          panel.grid.major = element_line(colour="grey",),panel.grid.minor = element_blank(),
          panel.background = element_blank(),legend.position="none")+
    scale_fill_manual(values=c("steelblue","red"))
  title=paste(ran*(i-1),"","\u2264<",ran*i, "",sep="")
  b=b+ggtitle(title)
  assign(paste("fig",i,sep=""),b)

}




#combine Fig. 3A, B, and C
g=ggarrange(ggarrange(fig1,fig2,fig3,fig4,fig5,fig6,ncol=3,nrow=2),nrow = 2, heights=c(1.33,1),ggarrange(fig2b,fig2c,ncol=2,labels=c("B","C")),labels=c("A",""))
g
# ggsave("fig2-2022.svg",plot=g,width=170,height=212.5,unit="mm",dpi=500)


### 2.3. Effects of initial orientation and predator speed on turn angle || (Fig.4-fig.sup.1)-----

#effects of io and pred.sp on turn angle in away response
fit=gamm4(ea~s(io)+s(pred.sp),random = ~ (1|fish),data=dfcon)
fit
anova(fit$gam)
anova(fit$mer)

fit=gamm4(ea~s(io),random = ~ (1|fish),data=dfcon)
xx=data.frame(io=seq(0,180,by=0.01))
predcon=predict(fit$gam,xx,se.fit=TRUE, type="response")


#effects of io and pred.sp on turn angle in toward response
fit=gamm4(ea~s(io)+s(pred.sp),random = ~ (1|fish),data=dfips)
fit
anova(fit$gam)
anova(fit$mer)

fit=gamm4(ea~s(io),random = ~ (1|fish),data=dfips)
xx=data.frame(io=seq(0,180,by=0.01))
predips=predict(fit$gam,xx,se.fit=TRUE, type="response")

#plot IO-turn angle relationship (Fig.3-fig.sup.2)
pred2=data.frame(eacon=as.numeric(predcon$fit),eaips=as.numeric(predips$fit),
                 io=as.numeric(xx$io),
                 Pu95con=as.numeric(predcon$fit + 1.96),
                 Pd95con=as.numeric(predcon$fit - 1.96),
                 Pu95ips=as.numeric(predips$fit + 1.96),
                 Pd95ips=as.numeric(predips$fit - 1.96))

ggplot()+geom_point(data=df3, aes(x=io,y=ea,fill=dir),size=1.5,shape=21)+ylab(tatext)+xlab(iotext)+
  geom_line(data=pred2,aes(x=io,y=eacon),size=1,linetype="solid")+
  geom_line(data=pred2,aes(x=io,y=eaips),size=1,linetype="dotted")+
  theme(axis.line=element_line(colour="black"),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_brewer(palette="Set1")+
  scale_x_continuous(limits=c(0,180),breaks=seq(0,180,by=60))+
  scale_y_continuous(limits=c(-150,180),breaks=seq(-180,330,by=60))


### 2.4. Gaussian mixture model analysis (Fig.3-source data 1)-----

#load data
et=dfemp$et

#fit 1- to 9-component Gaussian mixture distributions
result=Mclust(et)

#calculate AIC values
Aic1=-result$BIC[1]+2*(2-log(result$n))
Aic2=-result$BIC[2,2]+5*(2-log(result$n))
Aic3=-result$BIC[3,2]+8*(2-log(result$n))
Aic4=-result$BIC[4,2]+11*(2-log(result$n))
Aic5=-result$BIC[5,2]+14*(2-log(result$n))
Aic6=-result$BIC[6,2]+17*(2-log(result$n))
Aic7=-result$BIC[7,2]+20*(2-log(result$n))
Aic8=-result$BIC[8,2]+23*(2-log(result$n))
Aic9 =-result$BIC[9,2]+26*(2-log(result$n))

Aic=c(Aic1,Aic2,Aic3,Aic4,Aic5,Aic6,Aic7,Aic8,Aic9)
num=1:9
peaks=c(num)
dfmix=data.frame(peaks,AIC=Aic)
sortlist.Aic=order(dfmix$AIC)
dfmix = dfmix[sortlist.Aic,]
dfmix$deltaAIC=dfmix$AIC-min(dfmix$AIC)
dfmix #best model is the 3-component Gaussian mixture distribution
result=Mclust(et,G=3)
result$parameters


### 2.5. Effect of predator speed on ET and turn angle || variability-----

#load data
df4=dfemp

#categorize by speed at the onset of escape, and remove middle value
sp1=quantile(df4$pred.sp,0.3333333333333)
sp2=quantile(df4$pred.sp,0.6666666666666)
df4$pred.sp3=as.character(df4$pred.sp)
df4[df4$pred.sp<sp1,]$pred.sp3="s"
df4[df4$pred.sp>=sp1 & df4$pred.sp<sp2,]$pred.sp3="m"
df4[df4$pred.sp>=sp2,]$pred.sp3="f"
df4=df4[df4$pred.sp3=="s"|df4$pred.sp3=="f",]

#Leven's test in all 30 deg. initial orientation bins 
df30=df4[abs(df4$io)<30,]
boxplot(abs(ea)~pred.sp3,df30)
print(levene.test(y=abs(df30$ea),group=df30$pred.sp3))  
boxplot(et~pred.sp3,df30)
print(levene.test(y=df30$et,group=df30$pred.sp3))  

df30=df4[abs(df4$io)>=30 & abs(df4$io)<60,]
boxplot(abs(ea)~pred.sp3,df30)
print(levene.test(y=abs(df30$ea),group=df30$pred.sp3))  
boxplot(et~pred.sp3,df30)
print(levene.test(y=df30$et,group=df30$pred.sp3)) 

df30=df4[abs(df4$io)>=60 & abs(df4$io)<90,]
boxplot(abs(ea)~pred.sp3,df30)
print(levene.test(y=abs(df30$ea),group=df30$pred.sp3)) 
boxplot(et~pred.sp3,df30)
print(levene.test(y=df30$et,group=df30$pred.sp3)) 

df30=df4[abs(df4$io)>=90 & abs(df4$io)<120,]
boxplot(abs(ea)~pred.sp3,df30)
print(levene.test(y=abs(df30$ea),group=df30$pred.sp3)) 
boxplot(et~pred.sp3,df30)
print(levene.test(y=df30$et,group=df30$pred.sp3)) 

df30=df4[abs(df4$io)>=120 & abs(df4$io)<150,]
boxplot(abs(ea)~pred.sp3,df30)
print(levene.test(y=abs(df30$ea),group=df30$pred.sp3)) 
boxplot(et~pred.sp3,df30)
print(levene.test(y=df30$et,group=df30$pred.sp3)) 

df30=df4[abs(df4$io)>=150,]
boxplot(abs(ea)~pred.sp3,df30)
print(levene.test(y=abs(df30$ea),group=df30$pred.sp3)) 
boxplot(et~pred.sp3,df30)
print(levene.test(y=df30$et,group=df30$pred.sp3)) 



#### 3. Determination of parameter values-----

### 3.1. Determination of the D1 value (Fig.4-fig.sup.1)-----

#load data
dfemp=read.csv("Dataset1.csv")
dfttrsp=read.csv("Dataset2.csv")
dfttr=read.csv("Dataset3.csv")
dfemp=data.frame(ea=abs(dfemp$ea),fish=dfemp$fish,id=dfemp$id, dfttrsp)

###########choose the || range from the followings
#||<30 deg.
dfemp=dfemp[dfemp$ea<30,]
#30<=||<60 deg.
dfemp=dfemp[dfemp$ea>=30 & dfemp$ea<60,]
#60<=||<90 deg.
dfemp=dfemp[dfemp$ea>=60 & dfemp$ea<90,]
#||>=90 deg.
dfemp=dfemp[dfemp$ea>=90,]
##########the end of choosing range 

#adjust the data frame and calculate the mean speed per fish
dfcum=dfemp %>%  tidyr::gather (key=dis,value=sp,-ea,-fish,-id)
dfcum$dis=factor(dfcum$dis, 
                 levels = c("X1","X2","X3","X4","X5","X6","X7","X8","X9","X10"))

dfdifffish=dfcum %>%  dplyr::group_by(fish,dis) %>% 
  dplyr::summarise_each(dplyr::funs(mean))

#convert the distance (categorical) into a numeric value
extrnum <- str_extract_all(dfdifffish$dis, "[0-9.]+")
extnumstr <- sapply(extrnum,as.numeric)
dfdifffish$dis.n=extnumstr*3-1.5
dfdifffish$sp=dfdifffish$sp/1000

#Fig.4-fig.sup.1A~D
ggplot(dfdifffish,aes(x=dis.n,y=sp,group=fish))+geom_line()+geom_point(shape=21)+xlab("Displacement (mm)")+ylab(vel)+
  theme(axis.line=element_line(colour="black"),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_x_continuous(limits=c(0,30))+
  scale_y_continuous(limits=c(0,2.3))


#please use the following code to save the mean value of each || bin for making Fig.4-fig.sup.1E
#dfmean.l=data.frame(dis.n=seq(1.5,28.5,3),sp=tapply(dfdifffish$sp,dfdifffish$dis,mean)[1:10])

#multiple comparisons of the mean velocity between different 3-mm intervals in each || bin
pairwise.t.test(dfdifffish$sp, dfdifffish$dis, paired=T, p.adj = "bonf")


### 3.2. Effect of || on the time for displacement (and velocity) of 0-15 mm and 15-30mm-----

#load data
dfemp=read.csv("Dataset1.csv")
dfttrsp=read.csv("Dataset2.csv")
dfttr=read.csv("Dataset3.csv")
dfemp2=data.frame(ea=abs(dfemp$ea),dfttr,fish=dfemp$fish,id=dfemp$id)

dfemp2$ttrf=dfemp2$X5 #time for displacement of 0-15 mm
dfemp2$ttrrest=dfemp2$X10-dfemp2$X5 #time for displacement of 15-30 mm


#effect of || on the time for displacement of 0-15 mm, tested using GAMM
fit=gamm4(ttrf~s(ea),random = ~ (1|fish),data=dfemp2)
fit
anova(fit$gam)
anova(fit$mer)

#effect of || on the time for displacement of 15-30 mm, tested using GAMM
fit=gamm4(ttrrest~s(ea),random = ~ (1|fish),data=dfemp2)
fit
anova(fit$gam)
anova(fit$mer)

#effect of || on speed during the displacement of 0-15 mm, tested using GAMM 
fit=gamm4(I(15/ttrf/1000)~s(ea),random = ~ (1|fish),data=dfemp2)
fit
anova(fit$gam)
anova(fit$mer)

#effect of || on speed during the displacement of 15-30 mm, tested using GAMM 
fit=gamm4(I(15/ttrrest/1000)~s(ea),random = ~ (1|fish),data=dfemp2)
fit
anova(fit$gam)
anova(fit$mer)




### 3.3. Relationship between the turn angle || and time-distance variables (Fig. 4) -----
### 3.3.1. T1(||), the relationship between turn angle || and the time required for a displacement of 15 mm (Fig. 4A)-----

#load data
dfemp=read.csv("Dataset1.csv")
dfemp$ea=abs(dfemp$ea)

#parameters estimated from the hierarchical Bayesian analysis (See source code 2 for details)
#note that the estimated parameter values are slightly different each time in the Bayesian analysis 
intercept=27.50617
slope1=-0.00050
slope2=0.14723
cp=44.14768

#GAMM curve
fit=gamm4(I(ttr*1000)~s(ea),random = ~ (1|fish),data=dfemp)
fit
anova(fit$gam)
anova(fit$mer)

xx=data.frame(ea=seq(min(dfemp$ea),max(dfemp$ea),by=0.01))
pred=predict(fit$gam,xx,se.fit=TRUE, type="response")

pred3=data.frame(ttr=as.numeric(pred$fit),ea=as.numeric(xx$ea))


#piecewise linear regression lines
xx=seq(min(abs(dfemp$ea)),max(abs(dfemp$ea)),0.01)
piecewise=function(xx){
  if(xx<cp){
    yy=intercept+slope1*xx
  }else{
    yy=slope2*(xx-cp)+intercept+slope1*cp
  }
  return(yy)
}
yy=sapply(xx,piecewise)

pred2=data.frame(xx,yy)

#Fig. 4A
fig3a=ggplot()+geom_point(data=dfemp, aes(x=abs(ea),y=ttr*1000),size=1.5,shape=21)+
  ylab(ttrtext)+xlab(tatext2)+
  geom_line(data=pred3,aes(x=ea,y=ttr),size=0.5,linetype="solid",colour="red")+
  geom_line(data=pred2,aes(x=xx,y=yy),size=1,linetype="solid")+
  theme(axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_manual(values=c("red","steelblue"))+
  scale_x_continuous(limits=c(0,150),breaks=seq(0,180,by=30))


### 3.3.2. The relationship between turn angle || and the velocity after s1 turn (Fig. 4B)-----

#parameters estimated from the hierarchical Bayesian model
intercept=0.53667
slope1=0.00699
slope2=0.00025
cp=45.85190

#GAMM curve
fit=gamm4(I(V_s1)~s(ea),random = ~ (1|fish),data=dfemp)
fit
anova(fit$gam)
anova(fit$mer)

xx=data.frame(ea=seq(min(dfemp$ea),max(dfemp$ea),by=0.01))
pred=predict(fit$gam,xx,se.fit=TRUE, type="response")

pred3=data.frame(ttr=as.numeric(pred$fit),ea=as.numeric(xx$ea))


#piecewise linear regression lines
xx=seq(min(abs(dfemp$ea)),max(abs(dfemp$ea)),0.01)
piecewise=function(xx){
  if(xx<cp){
    yy=intercept+slope1*xx
  }else{
    yy=slope2*(xx-cp)+intercept+slope1*cp
  }
  return(yy)
}
yy=sapply(xx,piecewise)

pred2=data.frame(xx,yy)

#Fig. 4B
fig3b=ggplot()+geom_point(data=dfemp, aes(x=abs(ea),y=I(V_s1)),size=1.5,shape=21)+
  ylab("Velocity after s1 turn (m s-1)")+xlab(tatext2)+
  geom_line(data=pred3,aes(x=ea,y=ttr),size=0.5,linetype="solid",colour="red")+
  geom_line(data=pred2,aes(x=xx,y=yy),size=1,linetype="solid")+
  theme(axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_manual(values=c("red","steelblue"))+
  scale_x_continuous(limits=c(0,150),breaks=seq(0,180,by=30))+
  scale_y_continuous(limits=c(0.2,1.35),breaks=seq(0,1.5,by=0.5))

#combine Fig. 4A and 4B
ggarrange(fig3a,fig3b,ncol=2,labels=c("A","B"))
##see Source code 2 and 3 for including the 95% Bayesian credible intervals


### 3.4. Determination of Dattack and Upred-----
### 3.4.1. Optimization procedure (Fig.5-fig.sup.3)-----
#load the experimental data
dfempall=read.csv("Dataset1.csv")
dfemp=dfempall[,c("io","ea","et","dir","pred.sp")]
##in case of using predator's closing speed, replace pred.sp to pred.sp.close
#dfemp$pred.sp=dfempall$pred.sp.close

dfemp=na.omit(dfemp)

IO=0:180
ea2=abs(dfemp$ea)
eamin=min(ea2)
eamax=max(ea2)

#load the parameters for T1()
slope1=-0.00050
slope2=0.14723
intercept=27.50617

#for the model without T1(), use the following paramters
#slope1=0
#slope2=0
#intercept=30.64179

#for the model without Dattack, use the following parameters
#Dattack = Inf


#input parameters except for Dattack (Dattack) and Upred into the geometric model and find the optimal set of Dattack and Upred
rankdist=function(Dattack,Upred){ # for the model without Dattack, replace this line with "rankdist=function(Upred){"
  df=predict.ET1(IO,Uprey = 1042,Upred=Upred,slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=Dattack,eamin=round(min(abs(dfemp$ea))-1),eamax=round(max(abs(dfemp$ea))), Rdevice=199)
  
  find=function(a,b){
    return(df[df$io==a & df$et==b,]$d_rank)
  }
  rankdata=mapply(find, round(dfemp$io),round(dfemp$et))
  rankdata[is.na(rankdata)]=max(rankdata,na.rm=T)+1
  rankind=1-rankdata/max(rankdata)
  rank.mean=mean(rankind)
  return(rank.mean)
}

###in case when the empirical Upred data are used
#input parameters except for Dattack into the geometric model and find the optimal set of Dattack and Upred

# 
# rankdist=function(Dattack){ # for the model without Dattack, replace this line with "rankdist=function(Upred){"
#     rankdata=numeric()
#     for(i in 1:nrow(dfemp)){
#     df=predict.ET1(IO,Uprey = 1042,Upred=dfemp$pred.sp[i]*1000,slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=Dattack,eamin=round(min(abs(dfemp$ea))-1),eamax=round(max(abs(dfemp$ea))), Rdevice=199)
#     rankdata[i] = df[df$io==round(dfemp$io[i]) & df$et==round(dfemp$et[i]),]$d_rank
#     }
#     rankdata[is.na(rankdata)]=max(rankdata,na.rm=T)+1
#     rankind=1-rankdata/max(rankdata)
#     #hist(rankind)
#     rank.mean=mean(rankind)
#     return(rank.mean)
#   
#   }





fntime <- function(x) {
  x1 <- x[1]
  x2 = x[2]
  rankdist(Upred=x1, Dattack=x2)
}

#for the model without Dattack, use the following function,
#fntime <- function(x) {
#  x1 <- x[1]
#  rankdist(Upred=x1)
#}


# #for the model using empirical Upred, use the following function,
# fntime <- function(x) {
#   x1 <- x[1]
#   rankdist(Dattack=x1)
# }



#optimize the parameters with different initial values

#for the model without Dattack, use the following code
#opt=optimize(fntime, c(0,10000))


#for the model without Upred (using empirical Upred), use the following code
#opt=optimize(fntime, c(0,100)) 
#results: 
#$minimum
#[1] 28.52604
#$objective
#[1] 0.2061621
#opt$minimum=28.52604

#using closing speed 
# $minimum
# [1] 24.89276
# 
# $objective
# [1] 0.2133387
#opt$minimum=24.89276


opt1=optim(par=c(1000,50),fntime, control=list(fnscale=1),method="Nelder-Mead")
opt1
opt2=optim(par=c(2000,50),fntime, control=list(fnscale=1),method="Nelder-Mead")
opt2
opt3=optim(par=c(2000,20),fntime, control=list(fnscale=1),method="Nelder-Mead")
opt3
opt4=optim(par=c(1000,20),fntime, control=list(fnscale=1),method="Nelder-Mead")
opt4

opt=opt3  #choose the best one from above


#input all the parameters into the model
out=predict.ET1(IO,Uprey = 1042,Upred=opt$par[1],slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=opt$par[2],eamin=round(min(abs(dfemp$ea))-1),eamax=round(max(abs(dfemp$ea))),Rdevice=199)
#for the model without Dattack, use the following code
#out=predict.ET1(IO,Uprey = 1042,Upred=opt$minimum,slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=Inf,eamin=round(min(abs(dfemp$ea))-1),eamax=round(max(abs(dfemp$ea))),Rdevice=199)


#calculate ranking index
df=out
df$et=df$io+df$ea
rankemp=numeric()
for(i in 1:nrow(dfemp)){
  dftest=df[df$io==round(dfemp$io[i]),]
  if(length(dftest[dftest$et==round(dfemp$et[i]),]$d_rank)==0){
    rankemp[i]=NA
  }else{
    rankemp[i]=dftest[dftest$et==round(dfemp$et[i]),]$d_rank
  }
}


# #calculate ranking index for model using empirical Upred
# #df=out
# #df$et=df$io+df$ea
# rankemp=numeric()
# for(i in 1:nrow(dfemp)){
#   df=predict.ET1(IO,Uprey = 1042,Upred=dfemp$pred.sp[i]*1000,slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=28.52604,eamin=round(min(abs(dfemp$ea))-1),eamax=round(max(abs(dfemp$ea))),Rdevice=199)
#   df$et=df$io+df$ea
#   dftest=df[df$io==round(dfemp$io[i]),]
#   if(length(dftest[dftest$et==round(dfemp$et[i]),]$d_rank)==0){
#     rankemp[i]=NA
#   }else{
#     rankemp[i]=dftest[dftest$et==round(dfemp$et[i]),]$d_rank
#   }
# }





#fit truncated normal distribution to the ranking index

rankdata=1-rankemp/max(dftest$d_rank)
dtnorm0 <- function(x, mean, sd) {
  dtruncnorm(x,a=0,b=1,mean=mean,sd=sd)}
fit0 <- fitdistr(rankdata, dtnorm0, start = list(mean = mean(rankdata), sd = sd(rankdata)))

xx=seq(0,1,0.001)

if(fit0$estimate[1]<0){
  yy=dtnorm0(xx,mean=fit0$estimate[1],sd=fit0$estimate[2])*0.1
  
}else{
  fit0$estimate[1]=0
  dtnorm0 <- function(x, sd) {
    dtruncnorm(x,a=0,b=1,mean=0,sd=sd)}
  fit1 <- fitdistr(rankdata, dtnorm0, start = list(sd = sd(rankdata)))
  fit0$estimate[1]=0
  fit0$estimate[2]=fit1$estimate
  yy=dtnorm0(xx,sd=fit0$estimate[2])*0.1
}
rankdata=data.frame(rankdata)
pred3=data.frame(xx,yy)

#Fig.5-fig.sup.3
g=ggplot()+ geom_histogram(data=rankdata,binwidth=0.1,aes(x=rankdata,y=..count../sum(..count..)),
                               colour="black",size=0.25, boundary=0, fill="white")+
  ylab("Probability")+xlab("Ranking Index")+
  geom_line(data=pred3,aes(x=xx,y=yy),colour="black",size=1.5)+
  scale_x_continuous(limits=c(0,1),breaks=seq(0,1,by=0.2))+
  theme(axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_brewer(palette="Set1")
g

#ggsave("fig4-fig.sup.6-2.svg",plot=g,width=85,height=85,unit="mm",dpi=500)

### 3.4.2. Check the optimized values with the data from previously published predator-prey experiments (Fig.5-fig.sup.2)-----

#load data
orig1=read.csv("Dataset4.csv")
orig2=read.csv("Dataset5.csv")
orig3=read.csv("Dataset1.csv")

x.est1=34.72951 #estimated Dattack from optimal function
x.mean1=mean(orig1$Dattack)

x.est2=1.54324  #estimated Upred from optimal function
x.mean2=mean(orig2$Predator.speed) #real predator's speed

x.est3=1.54324  #estimated Upred from optimal function
x.mean3=mean(orig3$pred.sp)  #dummy predator speed at the onset of escape

x.est3=1.54324  #estimated Upred from optimal function
x.mean4=mean(orig3$pred.sp.close,na.rm=T) #dummy predator speed to cover the flight initiation distance

fig4sup1a=ggplot()+
  geom_histogram(data=orig1, binwidth=5,aes(x=Dattack,y=..count../sum(..count..)),colour="black",size=0.25,boundary=0)+
  geom_vline(xintercept = x.est1, linetype="dashed", 
             color = "blue", size=2)+
  geom_vline(xintercept = x.mean1, linetype="dotted", 
             color = "red", size=2)+
  theme(axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  xlab(expression(italic("D")[attack]~(mm)))+
  ylab("Relative frequency")


fig4sup1b=ggplot()+
  geom_histogram(data=orig2, binwidth=0.5, aes(x=Predator.speed,y=..count../sum(..count..)),colour="black",size=0.25,boundary=0)+
  geom_vline(xintercept = x.est2, linetype="dashed", 
             color = "blue", size=2)+
  geom_vline(xintercept = x.mean2, linetype="dotted", 
             color = "red", size=2)+
  theme(axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  xlab(expression(Predator~speed~(m~s^{-1})))+
  scale_x_continuous(limits=c(0,2.5),breaks=seq(0,2.5,by=0.5))+
  ylab("Relative frequency")


fig4sup1c=ggplot()+
  geom_histogram(data=orig3, binwidth=0.5, aes(x=pred.sp,y=..count../sum(..count..)),colour="black",size=0.25,boundary=0)+
  geom_vline(xintercept = x.est2, linetype="dashed", 
             color = "blue", size=2)+
  geom_vline(xintercept = x.mean3, linetype="dotted", 
             color = "red", size=2)+
  theme(axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  xlab(expression(Predator~speed~(m~s^{-1})))+
  scale_x_continuous(limits=c(0,2.5),breaks=seq(0,2.5,by=0.5))+
  ylab("Relative frequency")


fig4sup1d=ggplot()+
  geom_histogram(data=orig3, binwidth=0.5, aes(x=pred.sp.close,y=..count../sum(..count..)),colour="black",size=0.25,boundary=0)+
  geom_vline(xintercept = x.est2, linetype="dashed", 
             color = "blue", size=2)+
  geom_vline(xintercept = x.mean4, linetype="dotted", 
             color = "red", size=2)+
  theme(axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  xlab(expression(Predator~speed~(m~s^{-1})))+
  scale_x_continuous(limits=c(0,2.5),breaks=seq(0,2.5,by=0.5))+
  ylab("Relative frequency")

#combine Fig.5-fig.sup.2A, B, C, D
g=ggarrange(fig4sup1a,fig4sup1b,fig4sup1c,fig4sup1d,nrow=2,ncol=2,labels=c("A","B","C","D"))
g
#ggsave("fig4sup1.svg",plot=g,width=170,height=170,unit="mm",dpi=500)

#### 4. Comparison of model predictions and experimental data (Fig. 5; Fig.5-fig.sup.4~6; Table 3) -----

### 4.1. The circular graph for the theoretical time difference in each initial orientation bin (Fig. 5A)-----

IO=0:180
eamin=round(min(abs(dfemp$ea))-1)
eamax=round(max(abs(dfemp$ea)))
out=predict.ET1(IO,Uprey=1042,Upred=opt$par[1],slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=opt$par[2],eamin=eamin,eamax=eamax,Rdevice=199)
#for the model without Dattack, use the following code
#out=predict.ET1(IO,Uprey=1042,Upred=opt$minimum,slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=Inf,eamin=eamin,eamax=eamax,Rdevice=199)

dftheory=out

mx=0.003
mn=-0.05
ran=30

for(i in 1:180/ran){
  ioex=15+(i-1)*ran
  ea2=c((-180):(-eamax),(-eamin):eamin,eamax:180)
  df0=data.frame(dftheory[dftheory$io==ioex,])
  dfadd=data.frame(io=rep(ioex,length(ea2)),ea=ea2,dir=rep("con",length(ea2)),
                   y=rep((-10000),length(ea2)),d_dif=rep((-10000),length(ea2)),
                   T1=rep((-10000),length(ea2)),T2=rep((-10000),length(ea2)),
                   Tpred=rep(0,length(ea2)),d_rank=rep(360,length(ea2)),et=(ioex+ea2))
  df0=rbind(df0,dfadd)
  df0[df0$et<0,]$et=df0[df0$et<0,]$et+360
  df0=df0[order(df0$ea),]
  
  a=ggplot(df0, aes(x=et,y=d_dif))+geom_line(size=1)+
    coord_polar(start=270*pi/180,direction=-1)+
    scale_x_continuous(limits=c(0,360),breaks=seq(0,270,by=90),labels=c(0,90,180,-90))+
    scale_y_continuous(limits=c(-0.01,0),breaks=seq(-0.01,0.005,by=0.0025))+
    theme(plot.title=element_text(size=14,hjust=0.5),
          axis.line=element_blank(),axis.ticks.y=element_blank(),
          axis.title.x=element_blank(),axis.text.y=element_blank(),
          axis.title.y=element_blank(),axis.text.x=element_text(size=12),
          panel.grid.major = element_line(colour="grey",),panel.grid.minor = element_blank(),
          panel.background = element_blank(),legend.position="none")+
    scale_fill_brewer(palette="Spectral")
  title=paste("=",ioex,"",sep="")
  a=a+ggtitle(title)
  assign(paste("fig",i,sep=""),a)
}

### 4.2. The theoretical contour plots for the relationship between initial orientation and ET (Fig. 5B)-----

df=out
df$et=df$io+df$ea
df0.6=df[df$d_rank>quantile(df$d_rank,0.6),]
df0.7=df[df$d_rank>quantile(df$d_rank,0.7),]
df0.8=df[df$d_rank>quantile(df$d_rank,0.8),]
df0.9=df[df$d_rank>quantile(df$d_rank,0.9),]
df0.75=df[df$d_rank>quantile(df$d_rank,0.75),]

dflineC <- df %>%
  dplyr::arrange(io, y) %>%
  dplyr::group_by(io) %>%
  dplyr::filter(dir=="con")  %>%
  dplyr::filter(y==max(y))  %>%
  arrange(io,et,y)

dflineI <- df %>%
  dplyr::arrange(io, y) %>%
  dplyr::group_by(io) %>%
  dplyr::filter(dir=="ips")  %>%
  dplyr::filter(y==max(y))  %>%
  arrange(io,et,y)

dflineI=rbind(dflineC[1,],dflineI)
dflineC=dflineC[(-1),]

dfline=data.frame(io=0:180,ETcon=out[[6]],ETips=out[[8]])

fig4b=ggplot()+ylab(ettext)+xlab(iotext)+
  geom_point(data=df0.6,aes(x=io,y=ea+io),colour="#ffc0cb",size=0.5)+
  geom_point(data=df0.75,aes(x=io,y=ea+io),colour="orange",size=0.5)+
  geom_point(data=df0.9,aes(x=io,y=ea+io),colour="yellow",size=0.5)+
  geom_point(data=dfemp,aes(x=io,y=et),shape=21,size=1.5)+
  geom_line(data=dflineC,aes(x=io,y=et),size=1)+
  geom_line(data=dflineI,aes(x=io,y=et),size=1,linetype="dotted")+
  theme(axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_manual(values=c("red","steelblue"))+
  scale_x_continuous(limits=c(0,180),breaks=seq(0,180,by=60))+
  scale_y_continuous(limits=c(-150,240),breaks=seq(-180,330,by=60))



### 4.3. Monte Carlo simulation for the theoretical ET distributions (Fig. 5C)-----

#input parameters into the geometric model
out=predict.ET1(IO,Uprey = 1042,Upred=opt$par[1],slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=opt$par[2],eamin=round(min(abs(dfemp$ea))-1),eamax=round(max(abs(dfemp$ea))),Rdevice=199)
#for the model without Dattack, use the following code
#out=predict.ET1(IO,Uprey=1042,Upred=opt$minimum,slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=Inf,eamin=round(min(abs(dfemp$ea))-1),eamax=round(max(abs(dfemp$ea))),Rdevice=199)

dftheory=out
dftheory$et=dftheory$io+dftheory$ea
M=max(dftheory$d_rank)

#load fitted truncated normal distribution to the ranking index
mean.tn=fit0$estimate[1]
sd.tn=fit0$estimate[2]


etmontall=numeric() #theoretical ETs
Vval=numeric() #Kuiper's V
Pval=numeric() #P values of two-sample Kuiper test

#function for Monte Carlo Simulation
ETsim=function(io){
  ior=round(io)
  df2=dftheory[dftheory$io==ior,]
  rank=(1-rtruncnorm(1,a=0,b=1,mean=mean.tn,sd=sd.tn))*M
  num = which(abs(df2$d_rank-rank) == min(abs(df2$d_rank-rank)))
  if(length(num)==1){
    etmont=df2[num,]$et
  }else{
    num=sample(num,1)
    etmont=df2[num,]$et
  }
  return(etmont)
}

#run Monte Carlo simulation
for(l in 1:1000){
  etmont=numeric()
  ior=round(dfemp$io)
  etmont=sapply(ior,ETsim)
  data=circular(dfemp$et*pi/180)
  data2=circular(etmont*pi/180)
  fit=kuiper.2samp(data,data2)
  Vval[l]=fit$Kuiper.statistic
  Pval[l]=fit$p.value
  etmontall=c(etmontall,etmont)
}


# ##################in case where the empirical Upred is used-----------------
# 
# Dattack = 28.52604 #using predator speed at the onset of escape
# Dattack = 24.89276 #using predator closing speed to cover the distance toward the prey
# 
# #input parameters into the geometric model
# out=predict.ET1(IO,Uprey = 1042,Upred=dfemp$pred.sp[i]*1000,slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=Dattack,eamin=round(min(abs(dfemp$ea))-1),eamax=round(max(abs(dfemp$ea))),Rdevice=199)
# #for the model without Dattack, use the following code
# #out=predict.ET1(IO,Uprey=1042,Upred=opt$minimum,slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=Inf,eamin=round(min(abs(dfemp$ea))-1),eamax=round(max(abs(dfemp$ea))),Rdevice=199)
# 
# dftheory=out
# dftheory$et=dftheory$io+dftheory$ea
# M=max(dftheory$d_rank)
# 
# #load fitted truncated normal distribution to the ranking index
# mean.tn=fit0$estimate[1]
# sd.tn=fit0$estimate[2]
# 
# 
# etmontall=numeric() #theoretical ETs
# Vval=numeric() #Kuiper's V
# Pval=numeric() #P values of two-sample Kuiper test
# 
# 
# #run Monte Carlo simulation
# simN=1000
# dfetmont=  data.frame(N=seq(1:simN))
# etmontall=numeric()
# 
# for(i in 1:nrow(dfemp)){
#   dftheory=predict.ET1(IO,Uprey = 1042,Upred=dfemp$pred.sp[i]*1000,slope2=slope2/1000,slope1=slope1/1000,GC=44,intercept=intercept/1000,D1=15,Dwidth=18,Dattack=Dattack,eamin=round(min(abs(dfemp$ea))-1),eamax=round(max(abs(dfemp$ea))),Rdevice=199)
#   dftheory$et=dftheory$io+dftheory$ea
#   M=max(dftheory$d_rank)
#   ior=round(dfemp$io[i])
#   df2=dftheory[dftheory$io==ior,]
#   etmont=numeric()
#   for(l in 1:simN){
#       rank=(1-rtruncnorm(1,a=0,b=1,mean=mean.tn,sd=sd.tn))*M
#       num = which(abs(df2$d_rank-rank) == min(abs(df2$d_rank-rank)))
#       if(length(num)==1){
#       etmont[l]=df2[num,]$et
#       }else{
#       num=sample(num,1)
#       etmont[l]=df2[num,]$et
#       }
#   }
#   etmontall=c(etmontall,etmont)
#   dfetmont=data.frame(dfetmont,etmont)
# }
# dfetmont=dfetmont[,c(-1)]
# 
# 
# Vval=numeric()
# Pval=numeric()
# 
#   for(i in 1:nrow(dfetmont)){
#     data=circular(dfemp$et*pi/180)
#     data2=circular(dfetmont[i,]*pi/180)
#     fit=kuiper.2samp(data,data2)
#     Vval[i]=fit$Kuiper.statistic
#     Pval[i]=fit$p.value
#   }
#   
# #results of two-sample Kuiper test
# median(Vval)
# median(Pval)
# length(Pval[Pval>0.05])/length(Pval)
# 
# ################### the end of the case where the empirical Upred is used-----------------

#dataset obtained by Monte Carlo simulation
etmont=data.frame(et=etmontall,io=rep(dfemp$io,1000))
etmont$dir=etmont$io
etmont[etmont$io<etmont$et,]$dir="con"
etmont[etmont$io>etmont$et,]$dir="ips"
etmont$dir <- factor(etmont$dir,levels=c("ips","con"))
etmont$et.plot=etmont$et
etmont[etmont$et.plot<0,]$et.plot=etmont[etmont$et.plot<0,]$et.plot+360

#make the circular histogram
H=circular(etmont$et,type="angle",units="degrees",rotation="counter")
resx <- density(H,bw=50)

dfest=data.frame(x=seq(0,360,length=512),y=as.numeric(as.numeric(resx$y)))


mx=0.2
mn=-0.05


fig4c=ggplot()+
  coord_polar(start=270*pi/180,direction=-1)+
  geom_histogram(data=etmont, binwidth=15,aes(x=et.plot,y=..count../sum(..count..),
                                              fill=dir),colour="black",size=0.25,boundary=0)+
  geom_line(data=dfest,aes(x=x,y=y*0.1842865*1.5),colour="black",size=1)+
  scale_x_continuous(limits=c(0,360),breaks=seq(0,270,by=90),labels=c(0,90,180,-90))+
  scale_y_continuous(limits=c(mn,mx),breaks=seq(0,mx,by=0.1))+
  theme(axis.line=element_blank(),axis.ticks.y=element_blank(),
        axis.title.x=element_blank(),axis.text.y=element_blank(),
        axis.title.y=element_blank(),axis.text.x=element_text(size=12),
        panel.grid.major = element_line(colour="grey",),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_manual(values=c("red","steelblue"))

#ggsave("fig4-fig.sup.6-4.svg",plot=fig4c,width=85,height=85,unit="mm",dpi=500)

#combine Fig. 5A, B, and C
g=ggarrange(ggarrange(fig1,fig2,fig3,fig4,fig5,fig6,ncol=3,nrow=2),nrow = 2, heights=c(1.33,1),ggarrange(fig4b,fig4c,ncol=2,labels=c("B","C")),labels=c("A",""))
g
# ggsave("fig4-2022.svg",plot=g,width=170,height=212.5,unit="mm",dpi=500)

#results of two-sample Kuiper test
median(Vval)
median(Pval)
length(Pval[Pval>0.05])/length(Pval)


#### 5. Probability that the prey chooses the optimal vs. suboptimal ETs. (Fig. 6) -----
### 5.1. Graph needed for Fig. 6A (example of optimal ET advantage)-----
ioex=75 #specify IO
df0=dftheory[dftheory$io==ioex,]
eaadd=c((-180):(-eamax-1),(-eamin+1):(eamin-1),(eamax+1):180)
dfadd=data.frame(io=rep(ioex,length(eaadd)),ea=eaadd,dir=rep("con",length(eaadd)),
                 y=rep((-10000),length(eaadd)),d_dif=rep((-10000),length(eaadd)),
                 T1=rep((-10000),length(eaadd)),T2=rep((-10000),length(eaadd)),
                 Tpred=rep(0,length(eaadd)),d_rank=rep(360,length(eaadd)),et=(ioex+eaadd))

dfsub=rbind(df0,dfadd)

fig5a=ggplot(dfsub, aes(x=et,y=d_dif))+geom_line(size=1)+
  coord_polar(start=270*pi/180,direction=-1)+
  scale_x_continuous(limits=c(0,360),breaks=seq(0,270,by=90),labels = c(0,90,180,-90))+
  scale_y_continuous(limits=c(-0.01,0.003),breaks=seq(-0.01,0.005,by=0.003))+
  theme(plot.title=element_text(size=14,hjust=0.5),
        axis.line=element_blank(),axis.ticks.y=element_blank(),
        axis.title.x=element_blank(),axis.text.y=element_blank(),
        axis.title.y=element_blank(),axis.text.x=element_text(size=12),
        panel.grid.major = element_line(colour="grey",),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_brewer(palette="Spectral")
title=paste("=",ioex,"",sep="")
fig5a=fig5a+ggtitle(title)

### 5.2. Optimal and suboptimal choice analysis (Fig. 6C)-----
#load experimental data
dfemp=read.csv("Dataset1.csv")
dfemp$iocat3=dfemp$io
dfemp[which(dfemp$io<20),]$iocat3="0-20"
dfemp[which(dfemp$io>=20 & dfemp$io<40),]$iocat3="20-40"
dfemp[which(dfemp$io>=40 & dfemp$io<60),]$iocat3="40-60"
dfemp[which(dfemp$io>=60 & dfemp$io<80),]$iocat3="60-80"
dfemp[which(dfemp$io>=80 & dfemp$io<100),]$iocat3="80-100"
dfemp[which(dfemp$io>=100 & dfemp$io<120),]$iocat3="100-120"
dfemp[which(dfemp$io>=120 & dfemp$io<140),]$iocat3="120-140"
dfemp[which(dfemp$io>=140 & dfemp$io<160),]$iocat3="140-160"
dfemp[which(dfemp$io>=160),]$iocat3="160-180"

#input parameter values into the geometric model
intercept=27.50617/1000
slope1=-0.00050/1000
slope2=0.14723/1000
GC=44.14768
Upred=1543.24189
Dattack=34.72951
Uprey=1022
Dwidth=18
D1=15
IO=0:180
eamin=round(min(abs(dfemp$ea))-1)
eamax=round(max(abs(dfemp$ea)))

out=predict.ET1(IO,Uprey = 1022,Upred=Upred,slope2=slope2,slope1=slope1,GC=GC,intercept=intercept,D1=D1,Dwidth=Dwidth,Dattack=Dattack,eamin=round(min(abs(dfemp$ea))-1),eamax=round(max(abs(dfemp$ea))),Rdevice=199)


#calculate optimal ET advantage and choice of ET
optrange=35 #the range to include optimal-suboptimal choice analysis

dftheory=out
eaadd=c((-180):(-eamax-1),(-eamin+1):(eamin-1),(eamax+1):180)

choice=numeric() #whether real fish chose optimal (1) or suboptimal (0) ET
adv=numeric() #optimal ET advantage

for(i in 1:nrow(dfemp)){
  dftest=dfemp[i,]
  ioex=round(abs(dftest$io))
  df0=dftheory[dftheory$io==ioex,]
  dfadd=data.frame(io=rep(ioex,length(eaadd)),ea=eaadd,dir=rep("con",length(eaadd)),
                   y=rep((-10000),length(eaadd)),d_dif=rep((-10000),length(eaadd)),
                   T1=rep((-10000),length(eaadd)),T2=rep((-10000),length(eaadd)),
                   Tpred=rep(0,length(eaadd)),d_rank=rep(360,length(eaadd)),et=(ioex+eaadd))
  
  dfsub=rbind(df0,dfadd)
  dfsub=dfsub[order(dfsub$et),]
  a<-diff(dfsub$d_dif*1000)
  cross<-diff(a>0)
  opts<-which(cross==-1)+1
  dfopts=dfsub[opts,]
  dfopts=dfopts[order(dfopts$d_dif,decreasing=T),]
  dfopts=dfopts[1:2,]
  dfopts
  adv[i]=-diff(dfopts$d_dif)
  num <- which( abs(dfopts$et-dftest$et) == min(abs(dfopts$et-dftest$et)) )
  if(min(abs(dfopts$et-dftest$et))>optrange){
    choice[i]=NA   #choose none of the optimal nor suboptimal ET
  }else{
    if(num==1){
      choice[i]=1 #choose optimal ET
    }else{
      choice[i]=0 #choose suboptimal ET
    }
  }
}

#make data frame
df2=data.frame(adv=adv*1000,choice,io=abs(dfemp$io),et=dfemp$et,iocat=dfemp$iocat3,fish=as.factor(dfemp$fish))
df2=na.omit(df2)
df30=data.frame(choice=tapply(df2$choice,df2$iocat,mean),adv=tapply(df2$adv,df2$iocat,mean),adv.xsd=tapply(df2$adv,df2$iocat,sd),Ncat=tapply(df2$choice,df2$iocat,length))
df30$adv.ysd=sqrt(df30$choice*(1-df30$choice)/df30$Ncat)

#effect of optimal ET advantage on ET choice (optimal vs suboptimal ET)
fit=glmer(choice~adv+(1|fish),family="binomial",data=df2)
fitnull=glmer(choice~(1|fish),family="binomial",data=df2)
anova(fit,fitnull)

##make Fig. 6C

#estimate lines
est<-c(0.056720,0.221985)
x<-seq(min(df2$adv),max(df2$adv), by=0.01)
y<-est[1]+est[2]*x
yy<-1/(1+exp(-y))
dfest=data.frame(adv=x,choice=yy)


fig5c=ggplot()+geom_point(data=df30, aes(x=adv,y=choice),size=1.5,shape=19)+
  geom_text(data=df30, aes(x=adv,y=choice),
            label=rownames(df30), 
            nudge_x = -0.2, nudge_y = -0.02, 
            check_overlap = F
  )+
  ylab("Proportion of optimal ET")+xlab("Optimal ET advantage (ms)")+
  geom_line(data=dfest,aes(x=adv,y=choice),size=0.5,linetype="solid")+
  theme(axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_manual(values=c("red","steelblue"))

### 5.3. Relationship between initial orientation and optimal ET advantage (Fig. 6B)-----
#make initial orientation data ranging from 0 to 180
dfsim=data.frame(io=0:180,et=10:190)
choice=numeric() #whether real fish chose optimal (1) or suboptimal (0) ET
adv=numeric() #optimal ET advantage
for(i in 1:nrow(dfsim)){
  dftest=dfsim[i,]
  ioex=round(abs(dftest$io))
  df0=dftheory[dftheory$io==ioex,]
  dfadd=data.frame(io=rep(ioex,length(eaadd)),ea=eaadd,dir=rep("con",length(eaadd)),
                   y=rep((-10000),length(eaadd)),d_dif=rep((-10000),length(eaadd)),
                   T1=rep((-10000),length(eaadd)),T2=rep((-10000),length(eaadd)),
                   Tpred=rep(0,length(eaadd)),d_rank=rep(360,length(eaadd)),et=(ioex+eaadd))
  
  dfsub=rbind(df0,dfadd)
  dfsub=dfsub[order(dfsub$et),]
  a<-diff(dfsub$d_dif*1000)
  cross<-diff(a>0)
  opts<-which(cross==-1)+1
  dfopts=dfsub[opts,]
  dfopts=dfopts[order(dfopts$d_dif,decreasing=T),]
  dfopts=dfopts[1:2,]
  dfopts
  adv[i]=-diff(dfopts$d_dif)
  num <- which( abs(dfopts$et-dftest$et) == min(abs(dfopts$et-dftest$et)) )
  if(min(abs(dfopts$et-dftest$et))>optrange){
    choice[i]=NA   #choose none of the optimal nor suboptimal ET
  }else{
    if(num==1){
      choice[i]=1 #choose optimal ET
    }else{
      choice[i]=0 #choose suboptimal ET
    }
  }
}

fig5b=ggplot()+geom_line(data=data.frame(io=dfsim$io,adv=adv*1000), aes(x=io,y=adv),size=1)+
  ylab("Optimal ET advantage (ms)")+xlab(iotext)+
  theme(axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_manual(values=c("red","steelblue"))+
  scale_y_continuous(limits=c(0,8),breaks=seq(0,8,by=2))

#combine Fig. 6A, B, and C
ggarrange(fig5a,fig5b,fig5c,ncol=3,labels=c("A","B","C"))







#### 6. Monte Carlo simulation by changing the model parameters (Fig. 7; Fig.7-fig.sup.1~9)-----

#input parameters (change one or more parameters)
slope1=-0.00050
slope2=0.14723
intercept=27.50617
GC=44
Rdevice=199 
Uprey=1042
Upred=1543
Dattack=34.7
D1=15
Dwidth=18
IO=0:180 #uniform distribution in the initial orientation with 1 deg. increment
eamin=8
eamax=147


#additional parameters needed for the model modified from Corcoran's model
D0=130
Dtr=30 #note that the Dtr = Rturn (in the text) +  Dwidth
Dreact=70

#parameters for the ranking index curve
mean.tn=-0.3743127
sd.tn=0.3324662

#input parameters into the geometric model

#into the model modified from Domenici's model
out=predict.ET1(IO,Uprey=Uprey,Upred=Upred,slope2=slope2/1000,slope1=slope1/1000,
                GC=GC,intercept=intercept/1000,D1=D1,Dwidth=Dwidth,Dattack=Dattack,
                eamin=eamin,eamax=eamax,Rdevice=Rdevice)

#into the model modified from Corcoran's model
#out=predict.ET2(IO,Uprey = Uprey,Upred=Upred,slope2=slope2/1000,slope1=slope1/1000,
#                GC=GC,intercept=intercept/1000,D1=D1,Dwidth=Dwidth,Dattack=Dattack,
#                 eamin=eamin,eamax=eamax,Rdevice=Rdevice,Dreact=Dreact,D0=D0,Dtr=Dtr)


#model prediction for the relationship between initial orientation and ET
df=out
df$et=df$io+df$ea
df0.6=df[df$d_rank>quantile(df$d_rank,0.6),]
df0.7=df[df$d_rank>quantile(df$d_rank,0.7),]
df0.8=df[df$d_rank>quantile(df$d_rank,0.8),]
df0.9=df[df$d_rank>quantile(df$d_rank,0.9),]
df0.75=df[df$d_rank>quantile(df$d_rank,0.75),]

dflineC <- df %>%
  dplyr::arrange(io, y) %>%
  dplyr::group_by(io) %>%
  dplyr::filter(dir=="con")  %>%
  dplyr::filter(y==max(y))  %>%
  arrange(io,et,y)

dflineI <- df %>%
  dplyr::arrange(io, y) %>%
  dplyr::group_by(io) %>%
  dplyr::filter(dir=="ips")  %>%
  dplyr::filter(y==max(y))  %>%
  arrange(io,et,y)

dflineI=rbind(dflineC[1,],dflineI)
dflineC=dflineC[(-1),]


ggplot()+ylab(ettext)+xlab(iotext)+
  geom_point(data=df0.6,aes(x=io,y=ea+io),colour="#ffc0cb",size=0.5)+
  geom_point(data=df0.75,aes(x=io,y=ea+io),colour="orange",size=0.5)+
  geom_point(data=df0.9,aes(x=io,y=ea+io),colour="yellow",size=0.5)+
  geom_line(data=dflineC,aes(x=io,y=et),size=1)+
  geom_line(data=dflineI,aes(x=io,y=et),size=1,linetype="dotted")+
  theme(plot.title=element_text(size=14,hjust=0.5),
        axis.line=element_line(colour="black"),
        axis.title.x=element_text(size=14),
        axis.text.x=element_text(size=12),
        axis.title.y=element_text(size=14),
        axis.text.y=element_text(size=12),
        panel.grid.major = element_blank(),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_manual(values=c("red","steelblue"))+
  scale_x_continuous(limits=c(0,180),breaks=seq(0,180,by=60))+
  scale_y_continuous(limits=c(-150,240),breaks=seq(-180,330,by=60))


#Monte Carlo simulation for making theoretical circular histogram
dftheory=out
dftheory$et=dftheory$io+dftheory$ea

M=max(dftheory$d_rank)
etmontall=numeric()
Vval=numeric()
Pval=numeric()

ETsim=function(io){
  ior=round(io)
  df2=dftheory[dftheory$io==ior,]
  rank=(1-rtruncnorm(1,a=0,b=1,mean=mean.tn,sd=sd.tn))*M
  num = which(abs(df2$d_rank-rank) == min(abs(df2$d_rank-rank)))
  if(length(num)==1){
    etmont=df2[num,]$et
  }else{
    num=sample(num,1)
    etmont=df2[num,]$et
  }
  return(etmont)
}


n=100
for(l in 1:n){
  etmont=numeric()
  ior=round(IO)
  etmont=sapply(ior,ETsim)
  data=circular(IO*pi/180)
  data2=circular(IO*pi/180)
  etmontall=c(etmontall,etmont)
}

etmont=data.frame(et=etmontall,io=rep(IO,n))
etmont$dir=etmont$io
etmont[etmont$io<etmont$et,]$dir="con"
etmont[etmont$io>etmont$et,]$dir="ips"
etmont$dir <- factor(etmont$dir,levels=c("ips","con"))

#circular histogram for ET spanning 360 deg.
H=circular(etmont$et,type="angle",units="degrees",rotation="counter")
resx <- density(H,bw=50)

dfest=data.frame(x=seq(0,360,length=512),y=as.numeric(as.numeric(resx$y)))

mx=0.2
mn=-0.05

etmont[etmont$et<0,]$et=etmont[etmont$et<0,]$et+360


ggplot()+
  coord_polar(start=270*pi/180,direction=-1)+
  geom_histogram(data=etmont, binwidth=15,aes(x=et,y=..count../sum(..count..)),
                 colour="black",size=0.25,boundary=0)+
  geom_line(data=dfest,aes(x=x,y=y*0.1842865*1.5),colour="black",size=1)+
  scale_x_continuous(limits=c(0,360),breaks=seq(0,270,by=90),labels=c(0,90,180,-90))+
  scale_y_continuous(limits=c(mn,mx),breaks=seq(0,mx,by=0.1))+
  theme(plot.title=element_text(size=14,hjust=0.5),
        axis.line=element_blank(),axis.ticks.y=element_blank(),
        axis.title.x=element_blank(),axis.text.y=element_blank(),
        axis.title.y=element_blank(),axis.text.x=element_text(size=12),
        panel.grid.major = element_line(colour="grey",),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_manual(values=c("red","steelblue"))



#circular histrogram for ETsemi (0-180 deg. )  
etmont$et2=etmont$et
etmont[etmont$et2<0,]$et2=-etmont[etmont$et2<0,]$et2
etmont[etmont$et2>180,]$et2=360-etmont[etmont$et2>180,]$et2


H=circular(etmont$et2,type="angle",units="degrees",rotation="counter")
resx <- density(H,bw=50)

dfest=data.frame(x=seq(0,360,length=512),y=as.numeric(as.numeric(resx$y)))


mx=0.32
mn=-0.05


ggplot()+
  coord_polar(start=270*pi/180,direction=-1)+
  geom_histogram(data=etmont, binwidth=15,aes(x=et2,y=..count../sum(..count..)),
                 colour="black",size=0.25,boundary=0)+
  geom_line(data=dfest,aes(x=x,y=y*0.1842865*1.5),colour="black",size=1)+
  scale_x_continuous(limits=c(0,360),breaks=seq(0,270,by=90),labels=c(0,90,180,""))+
  scale_y_continuous(limits=c(mn,mx),breaks=seq(0,mx,by=0.1))+
  theme(plot.title=element_text(size=14,hjust=0.5),
        axis.line=element_blank(),axis.ticks.y=element_blank(),
        axis.title.x=element_blank(),axis.text.y=element_blank(),
        axis.title.y=element_blank(),axis.text.x=element_text(size=12),
        panel.grid.major = element_line(colour="grey",),panel.grid.minor = element_blank(),
        panel.background = element_blank(),legend.position="none")+
  scale_fill_manual(values=c("red","steelblue"))

